;;**********************************************************************
;;**********************************************************************
;;
;; LPF2.ASM
;;
;; Assembler source for 2 Pole Switched Capacitor Low Pass Filter
;;
;; Rev A 2001 June 12
;;
;; Copyright (c) 2001 Cypress Microsystems, Inc. All rights reserved.
;;
;;**********************************************************************
;;**********************************************************************
;;


export  LPF2_1_Start
export _LPF2_1_Start
export  LPF2_1_SetPower
export _LPF2_1_SetPower

export  LPF2_1_Stop
export _LPF2_1_Stop

;; -----------------------------------------------------------------
;;                         Register Definitions
;; -----------------------------------------------------------------
;; UM Cap value naming convention per Gregorian and Temes, p293 
;;
;;
;; * For a Mask/Val pair, this indicates that the value is
;;   determined by the user either through config-time parameteriza-
;;   tion or run-time manipulation.
;; BIT FIELD	         Mask/Val(hex)  Function
;; -----------------          ---- 	--------------------
;; FLOUT: OUTPUT HALF                 BUILT IN SW-CAP B BLOCK
;; FLOUT_CR0.OSZ              80/*  CB Cap Value (User parameter by table)
;; FLOUT_CR0.CLOCK_PHASE      04/0  PHASE_NORM
;; FLOUT_CR0.SIGN             02/0  POS 
;; FLOUT_CR0.CAP_SIZE         1F/*  A-cap: C3 Cap Value (User parameter)
;;
;; FLOUT_CR1.SCB_INPUT_MUX_A  E0/*  (Input selection by placement)		
;; FLOUT_CR1.CAP_SIZE         1F/0  B-cap: (set to 0 in LPF)
;; FLOUT_CR2.A_OUT            80/*  Output En. (User parameter, default DISABLE)
;;
;; FLOUT_CR2.COMP_OUT         40/0  DISABLED
;; FLOUT_CR2.AZ_PHASE         20/1  AZ_PH1
;; FLOUT_CR2.CAP_SIZE         1F/*  C-cap: C4 Cap Value (User parameter)
;;
;; FLOUT_CR3.REF_SELECT       C0/0  Ref to AGND
;; FLOUT_CR3.FB_CAP_SELECT    20/1  PH2_AZ_ENABLE
;; FLOUT_CR3.FB_CAP_GND       10/0  DISABLED
;; FLOUT_CR3.SBB              08/0  DISABLED
;; FLOUT_CR3.SCB_INPUT_MUX_B  04/1  IN2B
;; FLOUT_CR3.PWR_SELECT       03/0  Power OFF at start-up
;;
;; FLIN: INPUT HALF OF FILTER, BUILT IN SW-CAP A BLOCK
;; FLIN_CR0.OSZ               80/*  CA Cap Value (User parameter)
;; FLIN_CR0.CLOCK_PHASE       04/0  PHASE_NORM
;; FLIN_CR0.SIGN              02/1  NEG 
;; FLIN_CR0.CAP_SIZE          1F/*  A-cap: C1 Cap Value (User parameter)
;;
;; FLIN_CR1.SCB_INPUT_MUX_A_C E0/*  Input selection (User parameter)		
;; FLIN_CR1.CAP_SIZE          1F/*  B-cap: C2 Cap Value (User parameter)
;;
;; FLIN_CR2.A_OUT             80/0  DISABLED
;; FLIN_CR2.COMP_OUT          40/0  DISABLED
;; FLIN_CR2.AZ_PHASE          20/1  AZ_PH2
;; FLIN_CR2.CAP_SIZE          1F/0  C-cap: (set to 0 in LPF)
;;
;; FLIN_CR3.REF_SELECT        C0/0  Ref to AGND
;; FLIN_CR3.FB_CAP_SELECT     20/1  PH2_AZ_ENABLE
;; FLIN_CR3.FB_CAP_GND        10/0  DISABLED
;; FLIN_CR3.SCA_INPUT_MUX_B   0C/*  Conn to FLOUT-output (placement)
;; FLIN_CR3.PWR_SELECT        03/0  Power OFF at start-up

include "LPF2_1.inc"
include "m8c.inc"
POWERMASK: equ 03h                           ;Power field mask for CR3

;; -----------------------------------------------------------------
;; Start/SetPower - Applies power setting to the module's PSoC blocks
;; 
;; INPUTS: A contains the power setting 0=Off, 1=Low, 2=Med, 3=High
;; OUTUTS: None
;; -----------------------------------------------------------------

 LPF2_1_Start:
_LPF2_1_Start:
 LPF2_1_SetPower:
_LPF2_1_SetPower:

    and A, POWERMASK                         ; mask A to protect unchanged bits
    mov X, SP                                ; define temp store location
;
    push A                                   ; put power value in temp store
    mov A, reg[LPF2_1_FLIN_CR3]    ; read power value
    and A, ~POWERMASK                        ; clear power bits in A
    or  A, [X]                               ; combine power value with balance of reg.
    mov reg[LPF2_1_FLIN_CR3], A    ; move complete value back to register
;
    mov A, reg[LPF2_1_FLOUT_CR3]   ; read power value
    and A, ~POWERMASK                        ; clear power bits in A
    or  A, [X]                               ; combine power value with balance of reg.
    mov reg[LPF2_1_FLOUT_CR3], A   ; move complete value back to register
    pop A
    ret

;; -----------------------------------------------------------------
;; Stop - Cuts power to the user module.
;;
;; INPUTS:  None
;; OUTPUTS: None
;; -----------------------------------------------------------------
 LPF2_1_Stop:
_LPF2_1_Stop:

        
    and reg[LPF2_1_FLIN_CR3], ~POWERMASK
    and reg[LPF2_1_FLOUT_CR3], ~POWERMASK
    ret
